unit meddatadict;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FormEditingIntf, LCLIntf, LCLType, Graphics, mdldatadict,
  ProjectIntf, NewItemIntf, typinfo, db ;

type
 { TDDModelPageDescriptor }

 TDDModelPageDescriptor = Class(TFileDescPascalUnitWithResource)
 Public
   Constructor Create; override;
   Function GetLocalizedName : String; override;
   Function GetLocalizedDescription : String; override;
   Function GetInterfaceUsesSection : String; override;
   function GetImplementationSource(const Filename, SourceName, ResourceName: string): string; override;
 end;

 { TDDModelMediator }

 TDDModelMediator = class(TDesignerMediator)
 private
   FDDPage: TDDModelPage;
   procedure DrawField(F: TModelFieldDef; const ARect: TRect);
   procedure PaintBackground;
   procedure PaintComponent(AComponent: TModelComponent);
   procedure PaintTable(ATable: TModelTable);
   procedure PaintText(AText: TModelText);
   procedure SetTableColors(ACanvas: TCanvas);
   procedure SetTextColors(ACanvas: TCanvas);
 public
   // needed by the lazarus form editor
   class function CreateMediator(TheOwner, aForm: TComponent): TDesignerMediator; override;
   class function FormClass: TComponentClass; override;
   procedure GetBounds(AComponent: TComponent; out CurBounds: TRect); override;
   procedure SetBounds(AComponent: TComponent; NewBounds: TRect); override;
   procedure GetClientArea(AComponent: TComponent; out
           CurClientArea: TRect; out ScrollOffset: TPoint); override;
   procedure Paint; override;
   function ComponentIsIcon(AComponent: TComponent): boolean; override;
   function ParentAcceptsChild(Parent: TComponent;
               Child: TComponentClass): boolean; override;
 public
   procedure InvalidateRect(Sender: TObject; Const ARect: TRect; Erase: boolean);
 end;

procedure Register;

implementation

{ TDDModelPageDescriptor }

constructor TDDModelPageDescriptor.Create;
begin
  inherited Create;
  Name:='Daemon Mapper';
  ResourceClass:=TDDModelPage;
  UseCreateFormStatements:=False;
end;

function TDDModelPageDescriptor.GetLocalizedName: String;
begin
  Result:='Data dictionary page';
end;

function TDDModelPageDescriptor.GetLocalizedDescription: String;
begin
  Result:='One page in the data dictionary';
end;

function TDDModelPageDescriptor.GetInterfaceUsesSection: String;
begin
  Result:=inherited GetInterfaceUsesSection;
  Result:=Result+', mdldatadict';
end;

function TDDModelPageDescriptor.GetImplementationSource(const Filename,
  SourceName, ResourceName: string): string;


begin
  Result:=inherited GetImplementationSource(Filename, SourceName, ResourceName);
end;

{ TDDModelMediator }

class function TDDModelMediator.CreateMediator(TheOwner, aForm: TComponent
  ): TDesignerMediator;
var
  Mediator: TDDModelMediator;
  P : TDDModelPage;
begin
  Result:=inherited CreateMediator(TheOwner, aForm);
  Mediator:=TDDModelMediator(Result);
  P:=aForm as TDDModelPage;
  Mediator.FDDPage:=P;
  P.OnInvalidateRect:=@InvalidateRect;
end;

class function TDDModelMediator.FormClass: TComponentClass;
begin
  Result:=TDDModelPage;
end;

procedure TDDModelMediator.GetBounds(AComponent: TComponent; out
  CurBounds: TRect);

begin
  if Not (AComponent is TModelComponent) then
    inherited GetBounds(AComponent,CurBounds)
  else
    With TModelComponent(AComponent) do
      begin
      CurBounds:=Bounds(Left,Top,Width,Height);
      end;
end;

procedure TDDModelMediator.SetBounds(AComponent: TComponent; NewBounds: TRect);

Var
  MC : TModelComponent;

begin
  if Not (AComponent is TModelComponent) then
    inherited SetBounds(AComponent,NewBounds)
  else
    begin
    MC:=TModelComponent(AComponent);
    With NewBounds do
      MC.SetBounds(Left,Top,Right-Left,Bottom-Top);
    end
end;

procedure TDDModelMediator.GetClientArea(AComponent: TComponent; out
  CurClientArea: TRect; out ScrollOffset: TPoint);
Var
  MC : TModelComponent;
begin
  if Not (AComponent is TModelComponent) then
    inherited GetClientArea(AComponent,CurClientArea,ScrollOffset)
  else
    begin
    MC:=TModelComponent(AComponent);
    With MC do
      CurClientArea:=Rect(0,0,Width,Height);
    ScrollOffset:=Point(0,0);
    end;
end;

procedure TDDModelMediator.PaintBackground;

begin
  with LCLForm.Canvas do
    begin
    // fill background
    Brush.Style:=bsSolid;
    Brush.Color:=clLtGray;
    FillRect(0,0,FDDPage.Width,FDDPage.Height);
    end;
end;

procedure TDDModelMediator.SetTableColors(ACanvas : TCanvas);

begin
  ACanvas.Pen.Color:=clBlack;
  ACanvas.Pen.Style:=psSolid;
  ACanvas.Brush.Color:=clYellow;
  ACanvas.Brush.Style:=bsSolid;
end;

procedure TDDModelMediator.PaintTable(ATable : TModelTable);

Const
  LineHeight = 19;

Var
  R : TRect;
  I,T,M,Delta : Integer;

begin
  With ATable do
    begin
    SetTableColors(LCLForm.Canvas);
    LCLForm.Canvas.RoundRect(0,0,Width,Height,20,20);
    R.Top:=3;
    R.Left:=3;
    R.Bottom:=LineHeight;
    R.Right:=Width-3;
    LCLForm.Canvas.TextRect(R,3,3,ATable.TableName);
    LCLForm.Canvas.Line(0,LineHeight,Width,LineHeight);
    T:=20;
    I:=0;
    M:=Top+Height;
    Delta:=18;
    While (T<M) and (I<Fields.Count) do
      begin
      R.Top:=T;
      R.Bottom:=T+Delta;
      DrawField(Fields[i],R);
      T:=T+Delta;
      Inc(I);
      end;
    end
end;

procedure TDDModelMediator.DrawField(F : TModelFieldDef; Const ARect : TRect);

Var
  S,T : String;
  TS : TTextStyle;

begin
  // initialize textstyle.
  FillChar(TS,SizeOf(TS),0);
  TS.Alignment:=taRightJustify;
  TS.Layout:=tlTop;
  // Get texts.
  S:=F.FieldName;
  If (F.DomainName<>'') then
    T:=F.DomainName
  else
    begin
    T:=GetEnumName(TypeInfo(TFieldType),Ord(F.Fieldtype));
    Delete(T,1,2);
    end;
  // draw name
  LCLForm.Canvas.TextRect(ARect,ARect.Left+2,ARect.Top+2,S);
  // Draw type
  LCLForm.Canvas.TextRect(ARect,(ARect.Left+ARect.Right) div 2,ARect.Top+2,T,TS);
end;

procedure TDDModelMediator.SetTextColors(ACanvas : TCanvas);

begin
  ACanvas.Pen.Color:=clBlack;
  ACanvas.Pen.Style:=psSolid;
  ACanvas.Brush.Color:=clInfoBk;
  ACanvas.Brush.Style:=bsSolid;
end;

procedure TDDModelMediator.PaintText(AText : TModelText);

Const
  LineHeight = 19;

Var
  R : TRect;
  S : String;
  TS : TTextStyle;

begin
  SetTextColors(LCLForm.Canvas);
  LCLForm.Canvas.FillRect(0,0,AText.Width,AText.Height);
  R.Top:=3;
  R.Left:=3;
  R.Bottom:=LineHeight;
  R.Right:=AText.Width-3;
  LCLForm.Canvas.TextRect(R,3,3,AText.Caption);
  LCLForm.Canvas.Line(0,LineHeight,AText.Width,LineHeight);
  R.Top:=LineHeight+3;
  R.Bottom:=AText.Top+AText.Height-3;
  S:=AText.Text.Text;
  If (S<>'') then
    begin
    // initialize textstyle.
    FillChar(TS,SizeOf(TS),0);
    TS.Alignment:=taLeftJustify;
    TS.Layout:=tlTop;
    TS.Wordbreak:=True;
    TS.SingleLine:=False;
    LCLForm.Canvas.TextRect(R,R.Left,R.Top,S,TS);
    end
end;

procedure TDDModelMediator.PaintComponent(AComponent : TModelComponent);

begin
  if AComponent is TModelTable then
    PaintTable(TModelTable(AComponent))
  else if AComponent is TModelText then
    PaintText(TModelText(AComponent))
end;

procedure TDDModelMediator.Paint;

Var
   DC : HDC;
   I : Integer;
   Comp : TModelComponent;

begin
  PaintBackground;
  DC:=LCLForm.Canvas.Handle;
  For I:=0 to FDDPage.ComponentCount-1 do
    if FDDPage.Components[i] is TModelComponent then
      begin
      Comp:=TModelComponent(FDDPage.Components[i]);
      LCLForm.Canvas.SaveHandleState;
      try
        MoveWindowOrgEx(DC,Comp.Left,Comp.Top);
        if IntersectClipRect(DC,0,0,Comp.Width,Comp.Height)<>NullRegion then
          PaintComponent(Comp);
      finally
        LCLForm.Canvas.RestoreHandleState;
      end;
      end;
end;

function TDDModelMediator.ComponentIsIcon(AComponent: TComponent): boolean;
begin
  Result:=not (AComponent is TModelComponent);
end;

function TDDModelMediator.ParentAcceptsChild(Parent: TComponent;
  Child: TComponentClass): boolean;
begin
  Result:=(Parent is TDDModelPage);
end;

procedure TDDModelMediator.InvalidateRect(Sender: TObject; Const ARect: TRect;
  Erase: boolean);
begin
  if (LCLForm=nil) or (not LCLForm.HandleAllocated) then
     exit;
  LCLIntf.InvalidateRect(LCLForm.Handle,@ARect,Erase);
end;

Const
  SDataModeling = 'Data Modeling';

procedure Register;
begin
  FormEditingHook.RegisterDesignerMediator(TDDModelMediator);
  RegisterComponents(SDataModeling,[TModelTable,TModelText]);
  RegisterNewItemCategory(TNewIDEItemCategory.Create(SDataModeling));
  RegisterProjectFileDescriptor(TDDModelPageDescriptor.Create,SDataModeling);
end;

end.

